#!/usr/bin/perl 

use lib '/usr/sausalito/perl';
use CCE;

my $cce = new CCE(Namespace => 'Time');
$cce->connectfd();

my $localtime = '/etc/localtime';
my $oid = $cce->event_oid();
my $time_obj = $cce->event_object();
my $old = $cce->event_old();

# We can set the time and defer application to the OS using the
# deferCommit boolean flag.  Time chanes will be applied by a 
# constructor on CCE restart
unless($time_obj->{deferCommit})
{
	# set the timezone first
	my $zone = $time_obj->{timeZone};

	# Obnoxious glibc UTC sign swap
	if ($zone =~ /GMT\+\d+/) {
		$zone =~ s/\+/\-/;
	} elsif ($zone =~ /GMT\-\d+/) {
		$zone =~ s/\-/\+/;
	}

	my $link = '../usr/share/zoneinfo/' . $zone;
	if ($zone and (readlink($localtime) ne $link)) {
		unlink('/etc/localtime');
		symlink($link, '/etc/localtime');
	}

	# set the time if necessary. 
	my $time = $time_obj->{epochTime};

	if($old->{epochTime} && $time_obj->{epochOffset})
	{
		$time = $time + (time() - $time_obj->{epochOffset});
		$cce->set($oid, 'Time', {'epochOffset' => 0});
	}

	if (($time ne $old->{epochTime}) ||
	    ($old->{deferCommit})) 
	{
		
	    `/usr/sausalito/sbin/epochdate $time`;
	    # resync the hwclock with the time
	    system ("/sbin/hwclock --utc --systohc > /dev/null");
 	   unlink("/etc/adjtime"); # get rid of any clock skew
	    system ("/sbin/hwclock --utc --systohc > /dev/null");
	}

	# reload ntpd, if it's running, after the hw clock set
	Sauce::Service::service_run_init('ntpd', 'reload') if 
		($time_obj->{ntpAddress});
}

$cce->bye("SUCCESS");
exit 0;
